@@ -118,10 +118,8 @@ function class:declareOptions () end
 function class:_declareBaseOptions ()
    self:declareOption("class", function (_, name)
       if name then
-         if self._legacy then
-            self._name = name
-         elseif name ~= self._name then
-            SU.error("Cannot change class name after instantiation, derive a new class instead")
+         if name ~= self._name then
+            SU.error("Cannot change class after document instantiation")
          end
       end
       return self._name
@@ -228,7 +226,7 @@ function class:loadPackage (packname, options, reload)
          pack._create = function ()
             return current_instance
          end
-         pack(options, true)
+         pack(options, reload)
       else
          self.packages[packname] = pack(options, reload)
       end
