@@ -332,7 +332,7 @@ function utilities.error (message, isbug)
    utilities.warn(message, isbug)
    _skip_traceback_levels = 2
    io.stderr:flush()
-   SILE.outputter:finish() -- Only really useful from the REPL but no harm in trying
+   SILE.outputter:abort()
    SILE.scratch.caughterror = true
    error("", 2)
 end
@@ -860,4 +860,22 @@ function utilities.stripContentPos (content)
    return SU.ast.stripContentPos(content)
 end
 
+function utilities._avoid_base_class_use (obj)
+   if type(obj) == "table" and obj._name and obj._name == "base" then
+      local type_group = "SILE." .. obj.type .. "s"
+      SU.deprecated(type_group .. ".base", type_group .. ".default", "0.15.11", "0.16.0")
+   end
+end
+
+-- On demand loader, allows modules to be loaded into a specific scope only if/when accessed
+function utilities._module_loader (scope)
+   return setmetatable({}, {
+      __index = function (self, key)
+         local m = require(("%s.%s"):format(scope, key))
+         self[key] = m
+         return m
+      end,
+   })
+end
+
 return utilities
