@@ -2,19 +2,18 @@
 -- @interfaces inputters
 
 local _deprecated = [[
-  You appear to be using a document class '%s' programmed for SILE <= v0.12.5.
-  This system was refactored in v0.13.0 and the shims trying to make it
-  work temporarily without refactoring your classes have been removed
-  in v0.14.0. Please see v0.13.0 release notes for help.
+   You appear to be using a document class '%s' programmed for SILE <= v0.12.5.
+   This system was refactored in v0.13.0 and the shims trying to make it
+   work temporarily without refactoring your classes have been removed
+   in v0.14.0. Please see v0.13.0 release notes for help.
 ]]
 
 local inputter = pl.class()
 inputter.type = "inputter"
 inputter._name = "base"
 
-inputter._docclass = nil
-
 function inputter:_init (options)
+   SU._avoid_base_class_use(self)
    self.options = options or {}
 end
 
@@ -27,11 +26,12 @@ function inputter:classInit (options)
    end
    class = SILE.input.class or class or options.class or "plain"
    options.class = nil -- don't pass already consumed class option to constructor
-   constructor = self._docclass or constructor or SILE.require(class, "classes", true)
+   constructor = constructor or SILE.require(class, "classes", true)
    if constructor.id then
       SU.deprecated("std.object", "pl.class", "0.13.0", "0.14.0", string.format(_deprecated, constructor.id))
    end
-   SILE.documentState.documentClass = constructor(options)
+   -- Note SILE.documentState.documentClass is set by the instance's own :_post_init()
+   constructor(options)
 end
 
 function inputter:requireClass (tree)
@@ -48,11 +48,15 @@ end
 function inputter:process (doc)
    -- Input parsers can already return multiple ASTs, but so far we only process one
    local tree = self:parse(doc)[1]
+   if SU.debugging("inputter") and SU.debugging("ast") then
+      SU.debug("inputter", "Dumping AST tree before processing...\n")
+      SU.dump(tree)
+   end
    self:requireClass(tree)
    return SILE.process(tree)
 end
 
-function inputter.findInTree (_, tree, command)
+function inputter:findInTree (tree, command)
    SU.deprecated("SILE.inputter:findInTree", "SU.ast.findInTree", "0.15.0", "0.17.0")
    return SU.ast.findInTree(tree, command)
 end
@@ -63,7 +67,7 @@ local function process_ambles (ambles)
          SILE.processFile(amble)
       elseif type(amble) == "function" then
          SU.warn(
-            "Passing functions as pre/postambles is not officially sactioned and may go away without being marked as a breaking change."
+            "Passing functions as pre/postambles is not officially sactioned and may go away without being marked as a breaking change"
          )
          amble()
       elseif type(amble) == "table" then
@@ -81,11 +85,11 @@ local function process_ambles (ambles)
    end
 end
 
-function inputter.preamble (_)
+function inputter:preamble ()
    process_ambles(SILE.input.preambles)
 end
 
-function inputter.postamble (_)
+function inputter:postamble ()
    process_ambles(SILE.input.postambles)
 end
 
