@@ -18,8 +18,8 @@ function inputter.appropriate (round, filename, doc)
    end
 end
 
-function inputter.parse (_, doc)
-   local result, err = load(doc)
+function inputter:parse (doc)
+   local result, err = load(doc, SILE.currentlyProcessingFile)
    if not result then
       SU.error(err)
    end
