@@ -52,7 +52,7 @@ local function extendCommand (name, func)
    end
 end
 
-function package.register (_, sourceExt, targetExt, command)
+function package:register (sourceExt, targetExt, command)
    table.insert(SILE.scratch.converters, {
       sourceExt = sourceExt,
       targetExt = targetExt,
@@ -60,7 +60,7 @@ function package.register (_, sourceExt, targetExt, command)
    })
 end
 
-function package.checkConverters (_, source)
+function package:checkConverters (source)
    local resolvedSrc = SILE.resolveFile(source) or SU.error("Couldn't find file " .. source)
    for _, converter in ipairs(SILE.scratch.converters) do
       local extLen = string.len(converter.sourceExt)
@@ -124,7 +124,6 @@ We do this by registering a converter with the \autodoc:command{\converters:regi
 
 \begin[type=autodoc:codeblock]{raw}
 \use[module=packages.converters]
-
 \converters:register[from=.gif,to=.jpg,command=convert $SOURCE $TARGET]
 \end{raw}
 
@@ -140,7 +139,6 @@ This trick also works for text files:
 
 \begin[type=autodoc:codeblock]{raw}
 \converters:register[from=.md, to=.sil, command=pandoc -o $TARGET $SOURCE]
-
 \include[src=document.md]
 \end{raw}
 \end{document}
