@@ -86,7 +86,7 @@ SILE.types.node.insertionlist.frame = nil
 
 function SILE.types.node.insertionlist:_init (spec)
    SILE.types.node.vbox._init(self, spec)
-   self.typesetter = SILE.typesetters.base()
+   self.typesetter = SILE.typesetters.default()
 end
 
 function SILE.types.node.insertionlist:__tostring ()
@@ -128,7 +128,7 @@ function SILE.types.node.insertion:__tostring ()
    return "I<" .. self.nodes[1] .. "...>"
 end
 
-function SILE.types.node.insertion.outputYourself (_) end
+function SILE.types.node.insertion:outputYourself () end
 
 -- And some utility methods to make the insertion processing code
 -- easier to read.
@@ -254,7 +254,7 @@ function package:_init ()
 
    --[[ Actually shrink the frame. --]]
 
-   SILE.insertions.commitShrinkage = function (_, classname)
+   function SILE.insertions.commitShrinkage (_, classname)
       local opts = SILE.scratch.insertions.classes[classname]
       local reduceList = opts["stealFrom"]
       local stealPosition = opts["steal-position"] or "bottom"
@@ -474,7 +474,7 @@ function package:_init ()
    self:export("insert", insert)
 end
 
-function package.declareSettings (_)
+function package:declareSettings ()
    SILE.settings:declare({
       parameter = "insertion.penalty",
       type = "integer",
@@ -487,7 +487,7 @@ package.documentation = [[
 \begin{document}
 The \autodoc:package{footnotes} package works by taking auxiliary material (the footnote content), shrinking the current frame and inserting it into the footnote frame.
 This is powered by the \autodoc:package{insertions} package; it doesn’t provide any user-visible SILE commands, but provides Lua functionality to other packages.
-TeX wizards may be interested to realise that insertions are implemented by an external add-on package, rather than being part of the SILE core.
+TeX wizards may be interested to realize that insertions are implemented by an external add-on package, rather than being part of the SILE core.
 \end{document}
 ]]
 
