@@ -1,13 +1,10 @@
 local nodefactory = require("types.node")
 local hb = require("justenoughharfbuzz")
 local ot = require("core.opentype-parser")
-local syms = require("packages.math.unicode-symbols")
+local atoms = require("packages.math.atoms")
 local mathvariants = require("packages.math.unicode-mathvariants")
 local convertMathVariantScript = mathvariants.convertMathVariantScript
 
-local atomType = syms.atomType
-local symbolDefaults = syms.symbolDefaults
-
 local elements = {}
 
 local mathMode = {
@@ -211,12 +208,17 @@ function elements.mbox:_init ()
    self.relY = SILE.types.length(0) -- y position relative to its parent box
    self.value = {}
    self.mode = mathMode.display
-   self.atom = atomType.ordinary
+   self.atom = atoms.types.ord
    local font = {
       family = SILE.settings:get("math.font.family"),
       size = SILE.settings:get("math.font.size"),
       style = SILE.settings:get("math.font.style"),
       weight = SILE.settings:get("math.font.weight"),
+      -- https://learn.microsoft.com/en-us/typography/opentype/spec/math#opentype-layout-tags-used-with-the-math-table
+      --   "Script tag to be used for features in math layout.
+      --   The only language system supported with this tag is the default language system."
+      -- Thus, needed for the ssty feature in superscript/subscript to work properly.
+      script = "math",
    }
    local filename = SILE.settings:get("math.font.filename")
    if filename and filename ~= "" then
@@ -225,15 +227,15 @@ function elements.mbox:_init ()
    self.font = SILE.font.loadDefaults(font)
 end
 
-function elements.mbox.styleChildren (_)
+function elements.mbox:styleChildren ()
    SU.error("styleChildren is a virtual function that need to be overridden by its child classes")
 end
 
-function elements.mbox.shape (_, _, _)
+function elements.mbox:shape (_, _)
    SU.error("shape is a virtual function that need to be overridden by its child classes")
 end
 
-function elements.mbox.output (_, _, _, _)
+function elements.mbox:output (_, _, _)
    SU.error("output is a virtual function that need to be overridden by its child classes")
 end
 
@@ -298,55 +300,99 @@ local spaceKind = {
    thick = "thick",
 }
 
--- Indexed by left atom
+-- Spacing table indexed by left atom, as in TeXbook p. 170.
+-- Notes
+--  - the "notScript" key is used to prevent spaces in script and scriptscript modes
+--    (= parenthesized non-zero value in The TeXbook's table).
+--  - Cases commented are as expected, just listed for clarity and completeness.
+--    (= no space i.e. 0 in in The TeXbook's table)
+--  - Cases marked as impossible are not expected to happen (= stars in the TeXbook):
+--    "... such cases never arise, because binary atoms must be preceded and followed
+--    by atoms compatible with the nature of binary operations."
+--    This must be understood with the context explained onp. 133:
+--     "... binary operations are treated as ordinary symbols if they don’t occur
+--     between two quantities that they can operate on." (a rule which notably helps
+--     addressing binary atoms used as unary operators.)
 local spacingRules = {
-   [atomType.ordinary] = {
-      [atomType.bigOperator] = { spaceKind.thin },
-      [atomType.binaryOperator] = { spaceKind.med, notScript = true },
-      [atomType.relationalOperator] = { spaceKind.thick, notScript = true },
-      [atomType.inner] = { spaceKind.thin, notScript = true },
+   [atoms.types.ord] = {
+      -- [atoms.types.ord] = nil
+      [atoms.types.op] = { spaceKind.thin },
+      [atoms.types.bin] = { spaceKind.med, notScript = true },
+      [atoms.types.rel] = { spaceKind.thick, notScript = true },
+      -- [atoms.types.open] = nil
+      -- [atoms.types.close] = nil
+      -- [atoms.types.punct] = nil
+      [atoms.types.inner] = { spaceKind.thin, notScript = true },
    },
-   [atomType.bigOperator] = {
-      [atomType.ordinary] = { spaceKind.thin },
-      [atomType.bigOperator] = { spaceKind.thin },
-      [atomType.relationalOperator] = { spaceKind.thick, notScript = true },
-      [atomType.inner] = { spaceKind.thin, notScript = true },
+   [atoms.types.op] = {
+      [atoms.types.ord] = { spaceKind.thin },
+      [atoms.types.op] = { spaceKind.thin },
+      [atoms.types.bin] = { impossible = true },
+      [atoms.types.rel] = { spaceKind.thick, notScript = true },
+      -- [atoms.types.open] = nil
+      -- [atoms.types.close] = nil
+      -- [atoms.types.punct] = nil
+      [atoms.types.inner] = { spaceKind.thin, notScript = true },
    },
-   [atomType.binaryOperator] = {
-      [atomType.ordinary] = { spaceKind.med, notScript = true },
-      [atomType.bigOperator] = { spaceKind.med, notScript = true },
-      [atomType.openingSymbol] = { spaceKind.med, notScript = true },
-      [atomType.inner] = { spaceKind.med, notScript = true },
+   [atoms.types.bin] = {
+      [atoms.types.ord] = { spaceKind.med, notScript = true },
+      [atoms.types.op] = { spaceKind.med, notScript = true },
+      [atoms.types.bin] = { impossible = true },
+      [atoms.types.rel] = { impossible = true },
+      [atoms.types.open] = { spaceKind.med, notScript = true },
+      [atoms.types.close] = { impossible = true },
+      [atoms.types.punct] = { impossible = true },
+      [atoms.types.inner] = { spaceKind.med, notScript = true },
    },
-   [atomType.relationalOperator] = {
-      [atomType.ordinary] = { spaceKind.thick, notScript = true },
-      [atomType.bigOperator] = { spaceKind.thick, notScript = true },
-      [atomType.openingSymbol] = { spaceKind.thick, notScript = true },
-      [atomType.inner] = { spaceKind.thick, notScript = true },
+   [atoms.types.rel] = {
+      [atoms.types.ord] = { spaceKind.thick, notScript = true },
+      [atoms.types.op] = { spaceKind.thick, notScript = true },
+      [atoms.types.bin] = { impossible = true },
+      -- [atoms.types.rel] = nil
+      [atoms.types.open] = { spaceKind.thick, notScript = true },
+      -- [atoms.types.close] = nil
+      -- [atoms.types.punct] = nil
+      [atoms.types.inner] = { spaceKind.thick, notScript = true },
    },
-   [atomType.closeSymbol] = {
-      [atomType.bigOperator] = { spaceKind.thin },
-      [atomType.binaryOperator] = { spaceKind.med, notScript = true },
-      [atomType.relationalOperator] = { spaceKind.thick, notScript = true },
-      [atomType.inner] = { spaceKind.thin, notScript = true },
+   [atoms.types.open] = {
+      -- [atoms.types.ord] = nil
+      -- [atoms.types.op] = nil
+      [atoms.types.bin] = { impossible = true },
+      -- [atoms.types.rel] = nil
+      -- [atoms.types.open] = nil
+      -- [atoms.types.close] = nil
+      -- [atoms.types.punct] = nil
+      -- [atoms.types.inner] = nil
    },
-   [atomType.punctuationSymbol] = {
-      [atomType.ordinary] = { spaceKind.thin, notScript = true },
-      [atomType.bigOperator] = { spaceKind.thin, notScript = true },
-      [atomType.relationalOperator] = { spaceKind.thin, notScript = true },
-      [atomType.openingSymbol] = { spaceKind.thin, notScript = true },
-      [atomType.closeSymbol] = { spaceKind.thin, notScript = true },
-      [atomType.punctuationSymbol] = { spaceKind.thin, notScript = true },
-      [atomType.inner] = { spaceKind.thin, notScript = true },
+   [atoms.types.close] = {
+      -- [atoms.types.ord] = nil
+      [atoms.types.op] = { spaceKind.thin },
+      [atoms.types.bin] = { spaceKind.med, notScript = true },
+      [atoms.types.rel] = { spaceKind.thick, notScript = true },
+      -- [atoms.types.open] = nil
+      -- [atoms.types.close] = nil
+      -- [atoms.types.punct] = nil
+      [atoms.types.inner] = { spaceKind.thin, notScript = true },
    },
-   [atomType.inner] = {
-      [atomType.ordinary] = { spaceKind.thin, notScript = true },
-      [atomType.bigOperator] = { spaceKind.thin },
-      [atomType.binaryOperator] = { spaceKind.med, notScript = true },
-      [atomType.relationalOperator] = { spaceKind.thick, notScript = true },
-      [atomType.openingSymbol] = { spaceKind.thin, notScript = true },
-      [atomType.punctuationSymbol] = { spaceKind.thin, notScript = true },
-      [atomType.inner] = { spaceKind.thin, notScript = true },
+   [atoms.types.punct] = {
+      [atoms.types.ord] = { spaceKind.thin, notScript = true },
+      [atoms.types.op] = { spaceKind.thin, notScript = true },
+      [atoms.types.bin] = { impossible = true },
+      [atoms.types.rel] = { spaceKind.thin, notScript = true },
+      [atoms.types.open] = { spaceKind.thin, notScript = true },
+      [atoms.types.close] = { spaceKind.thin, notScript = true },
+      [atoms.types.punct] = { spaceKind.thin, notScript = true },
+      [atoms.types.inner] = { spaceKind.thin, notScript = true },
+   },
+   [atoms.types.inner] = {
+      [atoms.types.ord] = { spaceKind.thin, notScript = true },
+      [atoms.types.op] = { spaceKind.thin },
+      [atoms.types.bin] = { spaceKind.med, notScript = true },
+      [atoms.types.rel] = { spaceKind.thick, notScript = true },
+      [atoms.types.open] = { spaceKind.thin, notScript = true },
+      [atoms.types.punct] = { spaceKind.thin, notScript = true },
+      -- [atoms.types.close] = nil
+      [atoms.types.inner] = { spaceKind.thin, notScript = true },
    },
 }
 
@@ -378,14 +424,54 @@ function elements.stackbox:styleChildren ()
    end
    if self.direction == "H" then
       -- Insert spaces according to the atom type, following Knuth's guidelines
-      -- in the TeXbook
+      -- in The TeXbook, p. 170 (amended with p. 133 for binary operators)
+      -- FIXME: This implementation is not using the atom form and the MathML logic (lspace/rspace).
+      -- (This is notably unsatisfactory for <mphantom> elements)
       local spaces = {}
+      if #self.children >= 1 then
+         -- An interpretation of the TeXbook p. 133 for binary operator exceptions:
+         -- A binary operator at the beginning of the expression is treated as an ordinary atom
+         -- (so as to be considered as a unary operator, without more context).
+         local v = self.children[1]
+         if v.atom == atoms.types.bin then
+            v.atom = atoms.types.ord
+         end
+      end
       for i = 1, #self.children - 1 do
          local v = self.children[i]
          local v2 = self.children[i + 1]
+         -- Handle re-wrapped paired open/close symbols
+         v = v.is_paired and v.children[#v.children] or v
+         v2 = v2.is_paired and v2.children[1] or v2
          if spacingRules[v.atom] and spacingRules[v.atom][v2.atom] then
             local rule = spacingRules[v.atom][v2.atom]
-            if not (rule.notScript and (isScriptMode(self.mode) or isScriptScriptMode(self.mode))) then
+            if rule.impossible then
+               -- Another interpretation of the TeXbook p. 133 for binary operator exceptions:
+               if v2.atom == atoms.types.bin then
+                  -- If a binary atom follows an atom that is not compatible with it, make it an ordinary.
+                  -- (so as to be conidered as a unary operator).
+                  -- Typical case: "a = -b" (ord rel bin ord), "a + -b" (ord bin bin ord)
+                  v2.atom = atoms.types.ord
+               else
+                  -- If a binary atom precedes an atom that is not compatible with it, make it an ordinary.
+                  -- Quite unusual case (bin, rel/close/punct) unlikely to happen in practice.
+                  -- (Not seen in 80+ test formulas)
+                  -- We might address it a bit late here, the preceding atom has already based its spacing
+                  -- on the binary atom... but this might not be a big deal.
+                  -- (i.e. rather than add an extra look-ahead just for this case).
+                  -- Artificial example: "a + = b" (ord bin rel ord)
+                  v.atom = atoms.types.ord
+               end
+               rule = spacingRules[v.atom][v2.atom]
+               if rule and rule.impossible then
+                  -- Should not occur if we did our table based on the TeXbook correctly?
+                  -- We can still handle it by ignoring the rule: no spacing sounds logical.
+                  -- But let's have a warning so it might be investigated further.
+                  SU.warn("Impossible spacing rule for (" .. v.atom .. ", " .. v2.atom .. "), please report this issue")
+                  rule = nil
+               end
+            end
+            if rule and not (rule.notScript and (isScriptMode(self.mode) or isScriptScriptMode(self.mode))) then
                spaces[i + 1] = rule[1]
             end
          end
@@ -423,7 +509,7 @@ function elements.stackbox:shape ()
       end
       -- Handle stretchy operators
       for _, elt in ipairs(self.children) do
-         if elt.is_a(elements.text) and elt.kind == "operator" and SU.boolean(elt.stretchy, false) then
+         if elt:is_a(elements.text) and elt.kind == "operator" and SU.boolean(elt.stretchy, false) then
             elt:_vertStretchyReshape(self.depth, self.height)
          end
       end
@@ -463,7 +549,7 @@ function elements.stackbox:outputYourself (typesetter, line)
    typesetter.frame:advanceWritingDirection(scaleWidth(self.width, line))
 end
 
-function elements.stackbox.output (_, _, _, _) end
+function elements.stackbox:output (_, _, _) end
 
 elements.phantom = pl.class(elements.stackbox) -- inherit from stackbox
 elements.phantom._type = "Phantom"
@@ -632,7 +718,7 @@ function elements.subscript:shape ()
    )
 end
 
-function elements.subscript.output (_, _, _, _) end
+function elements.subscript:output (_, _, _) end
 
 elements.underOver = pl.class(elements.subscript)
 elements.underOver._type = "UnderOver"
@@ -651,9 +737,41 @@ local function isNotEmpty (element)
    return element and (element:is_a(elements.terminal) or #element.children > 0)
 end
 
-function elements.underOver:_init (base, sub, sup)
+local function getAccentMode (mode)
+   -- Size unchanged but leave display mode
+   -- See MathML Core §3.4.3
+   if mode == mathMode.display then
+      return mathMode.text
+   end
+   if mode == mathMode.displayCramped then
+      return mathMode.textCramped
+   end
+   return mode
+end
+
+local function unwrapSingleElementMrow (elt)
+   -- CODE SMELL.
+   -- For \overset or \underset in LaTeX, MathML would use <mover> or <munder>.
+   -- It would need to inherit the base's atom type, especially if the later is an operator
+   -- (binary, relational etc.), which is a fairly common case, e.g.
+   --   \overset{R}{=} (equality with a R above the equal in some Ramanujan summations),
+   -- but we can't remove 1-element mrow's in the math typesetter, or have them inherit
+   -- their base's atom type here above, because it breaks tables for some reasons
+   -- that I couldn't figure out.
+   if elt:is_a(elements.stackbox) and elt.direction == "H" and #elt.children == 1 then
+      return unwrapSingleElementMrow(elt.children[1])
+   else
+      return elt
+   end
+end
+
+function elements.underOver:_init (attributes, base, sub, sup)
    elements.mbox._init(self)
+   base = unwrapSingleElementMrow(base)
    self.atom = base.atom
+   self.attributes = attributes or {}
+   self.attributes.accent = SU.boolean(self.attributes.accent, false)
+   self.attributes.accentunder = SU.boolean(self.attributes.accentunder, false)
    self.base = base
    self.sub = isNotEmpty(sub) and sub or nil
    self.sup = isNotEmpty(sup) and sup or nil
@@ -673,10 +791,10 @@ function elements.underOver:styleChildren ()
       self.base.mode = self.mode
    end
    if self.sub then
-      self.sub.mode = getSubscriptMode(self.mode)
+      self.sub.mode = self.attributes.accentunder and getAccentMode(self.mode) or getSubscriptMode(self.mode)
    end
    if self.sup then
-      self.sup.mode = getSuperscriptMode(self.mode)
+      self.sup.mode = self.attributes.accent and getAccentMode(self.mode) or getSuperscriptMode(self.mode)
    end
 end
 
@@ -694,14 +812,14 @@ function elements.underOver:_stretchyReshapeToBase (part)
    --   MathML3 "complex1" torture test: Maxwell's Equations (vectors in fractions)
    if #part.children == 0 then
       local elt = part
-      if elt.is_a(elements.text) and elt.kind == "operator" and SU.boolean(elt.stretchy, false) then
+      if elt:is_a(elements.text) and elt.kind == "operator" and SU.boolean(elt.stretchy, false) then
          elt:_horizStretchyReshape(self.base.width)
       end
    elseif part:is_a(elements.underOver) then
       -- Big assumption here: only considering one level of stacked under/over.
       local hasStretched = false
       for _, elt in ipairs(part.children) do
-         if elt.is_a(elements.text) and elt.kind == "operator" and SU.boolean(elt.stretchy, false) then
+         if elt:is_a(elements.text) and elt.kind == "operator" and SU.boolean(elt.stretchy, false) then
             local stretched = elt:_horizStretchyReshape(self.base.width)
             if stretched then
                hasStretched = true
@@ -718,7 +836,10 @@ function elements.underOver:_stretchyReshapeToBase (part)
 end
 
 function elements.underOver:shape ()
+   local constants = self:getMathMetrics().constants
+   local scaleDown = self:getScaleDown()
    local isMovableLimits = SU.boolean(self.base and self.base.movablelimits, false)
+   local itCorr = self:calculateItalicsCorrection() * scaleDown
    if not (self.mode == mathMode.display or self.mode == mathMode.displayCramped) and isMovableLimits then
       -- When the base is a movable limit, the under/over scripts are not placed under/over the base,
       -- but other to the right of it, when display mode is not used.
@@ -729,32 +850,54 @@ function elements.underOver:shape ()
       elements.subscript.shape(self)
       return
    end
-   local constants = self:getMathMetrics().constants
-   local scaleDown = self:getScaleDown()
    -- Determine relative Ys
    if self.base then
       self.base.relY = SILE.types.length(0)
    end
    if self.sub then
       self:_stretchyReshapeToBase(self.sub)
-      self.sub.relY = self.base.depth
-         + SILE.types.length(
-            math.max(
-               (self.sub.height + constants.lowerLimitGapMin * scaleDown):tonumber(),
-               constants.lowerLimitBaselineDropMin * scaleDown
+      -- TODO These rules are incomplete and even wrong if we were to fully implement MathML Core.
+      if self.attributes.accentunder then
+         self.sub.relY = self.base.depth
+            + SILE.types.length(
+               (self.sub.height + constants.lowerLimitGapMin * scaleDown):tonumber()
+               -- We assume that the accent is aligned on the base.
             )
-         )
+      else
+         self.sub.relY = self.base.depth
+            + SILE.types.length(
+               math.max(
+                  (self.sub.height + constants.lowerLimitGapMin * scaleDown):tonumber(),
+                  constants.lowerLimitBaselineDropMin * scaleDown
+               )
+            )
+      end
    end
    if self.sup then
       self:_stretchyReshapeToBase(self.sup)
-      self.sup.relY = 0
-         - self.base.height
-         - SILE.types.length(
-            math.max(
-               (constants.upperLimitGapMin * scaleDown + self.sup.depth):tonumber(),
-               constants.upperLimitBaselineRiseMin * scaleDown
+      -- TODO These rules are incomplete if we were to fully implement MathML Core.
+      if self.attributes.accent then
+         self.sup.relY = 0 - self.base.height
+         -- MathML Core wants to align on the accentBaseHeight...
+         local overShift = math.max(0, constants.accentBaseHeight * scaleDown - self.base.height:tonumber())
+         self.sup.relY = self.sup.relY - SILE.types.length(overShift)
+         -- HACK: .... but improperly dimensioned accents can overshoot the base glyph.
+         -- So we try some guesswork to correct this.
+         -- Typically some non-combining symbols are in this case...
+         local heuristics = 0.5 * constants.flattenedAccentBaseHeight + 0.5 * constants.accentBaseHeight
+         if self.sup.height > SILE.types.length(heuristics * scaleDown) then
+            self.sup.relY = self.sup.relY + SILE.types.length(constants.accentBaseHeight * scaleDown)
+         end
+      else
+         self.sup.relY = 0
+            - self.base.height
+            - SILE.types.length(
+               math.max(
+                  (constants.upperLimitGapMin * scaleDown + self.sup.depth):tonumber(),
+                  constants.upperLimitBaselineRiseMin * scaleDown
+               )
             )
-         )
+      end
    end
    -- Determine relative Xs based on widest symbol
    local widest, a, b
@@ -795,7 +938,6 @@ function elements.underOver:shape ()
    if b then
       b.relX = c - b.width / 2
    end
-   local itCorr = self:calculateItalicsCorrection() * scaleDown
    if self.sup then
       self.sup.relX = self.sup.relX + itCorr / 2
    end
@@ -829,7 +971,7 @@ function elements.underOver:calculateItalicsCorrection ()
          -- If this is a big operator, and we are in display style, then the
          -- base glyph may be bigger than the font size. We need to adjust the
          -- italic correction accordingly.
-         if self.base.atom == atomType.bigOperator and isDisplayMode(self.mode) then
+         if SU.boolean(self.base.largeop) and isDisplayMode(self.mode) then
             c = c * (self.base and self.base.font.size / self.font.size or 1.0)
          end
          return c
@@ -838,7 +980,7 @@ function elements.underOver:calculateItalicsCorrection ()
    return 0
 end
 
-function elements.underOver.output (_, _, _, _) end
+function elements.underOver:output (_, _, _) end
 
 -- terminal is the base class for leaf node
 elements.terminal = pl.class(elements.mbox)
@@ -848,9 +990,9 @@ function elements.terminal:_init ()
    elements.mbox._init(self)
 end
 
-function elements.terminal.styleChildren (_) end
+function elements.terminal:styleChildren () end
 
-function elements.terminal.shape (_) end
+function elements.terminal:shape () end
 
 elements.space = pl.class(elements.terminal)
 elements.space._type = "Space"
@@ -901,7 +1043,7 @@ function elements.space:shape ()
    self.depth = self.depth:absolute() * self:getScaleDown()
 end
 
-function elements.space.output (_) end
+function elements.space:output (_, _, _) end
 
 -- text node. For any actual text output
 elements.text = pl.class(elements.terminal)
@@ -947,6 +1089,17 @@ end
 
 function elements.text:shape ()
    self.font.size = self.font.size * self:getScaleDown()
+   if isScriptMode(self.mode) then
+      local scriptFeature = SILE.settings:get("math.font.script.feature")
+      if scriptFeature then
+         self.font.features = ("+%s=1"):format(scriptFeature)
+      end
+   elseif isScriptScriptMode(self.mode) then
+      local scriptFeature = SILE.settings:get("math.font.script.feature")
+      if scriptFeature then
+         self.font.features = ("+%s=2"):format(scriptFeature)
+      end
+   end
    local face = SILE.font.cache(self.font, SILE.shaper.getFace)
    local mathMetrics = self:getMathMetrics()
    local glyphs = SILE.shaper:shapeToken(self.text, self.font)
@@ -1020,7 +1173,7 @@ function elements.text:shape ()
    end
 end
 
-function elements.text.findClosestVariant (_, variants, requiredAdvance, currentAdvance)
+function elements.text:findClosestVariant (variants, requiredAdvance, currentAdvance)
    local closest
    local closestI
    local m = requiredAdvance - currentAdvance
@@ -1092,7 +1245,10 @@ end
 function elements.text:_vertStretchyReshape (depth, height)
    local hasStretched = self:_stretchyReshape(depth + height, true)
    if hasStretched then
-      -- HACK: see output routine
+      -- RESCALING HACK: see output routine
+      -- We only do it if the scaling logic found constructions on the vertical block axis.
+      -- It's a dirty hack until we properly implement assembly of glyphs in the case we couldn't
+      -- find a big enough variant.
       self.vertExpectedSz = height + depth
       self.vertScalingRatio = (depth + height):tonumber() / (self.height:tonumber() + self.depth:tonumber())
       self.height = height
@@ -1103,12 +1259,21 @@ end
 
 function elements.text:_horizStretchyReshape (width)
    local hasStretched = self:_stretchyReshape(width, false)
-   if hasStretched then
-      -- HACK: see output routine
-      self.horizScalingRatio = width:tonumber() / self.width:tonumber()
-      self.width = width
+   if not hasStretched and width:tonumber() < self.width:tonumber() then
+      -- Never shrink glyphs, it looks ugly
+      return false
    end
-   return hasStretched
+   -- But if stretching couldn't be done, it will be ugly anyway, so we will force
+   -- a re-scaling of the glyph.
+   -- (So it slightly different from the vertical case, 'cause MathML just has one stretchy
+   -- attribute, whether for stretching on the vertical (block) or horizontal (inline) axis,
+   -- and we cannot know which axis is meant unless we implement yet another mapping table
+   -- as the one in the MathML Core appendices. Frankly, how many non-normative appendices
+   -- do we need to implement MathML correctly?)
+   -- RESCALING HACK: see output routine
+   self.horizScalingRatio = width:tonumber() / self.width:tonumber()
+   self.width = width
+   return true
 end
 
 function elements.text:output (x, y, line)
@@ -1247,7 +1412,7 @@ local function newSubscript (spec)
 end
 
 local function newUnderOver (spec)
-   return elements.underOver(spec.base, spec.sub, spec.sup)
+   return elements.underOver(spec.attributes, spec.base, spec.sub, spec.sup)
 end
 
 -- TODO replace with penlight equivalent
@@ -1272,9 +1437,9 @@ function elements.mtr:styleChildren ()
    end
 end
 
-function elements.mtr.shape (_) end -- done by parent table
+function elements.mtr:shape () end -- done by parent table
 
-function elements.mtr.output (_) end
+function elements.mtr:output () end
 
 elements.table = pl.class(elements.mbox)
 elements.table._type = "table" -- TODO why case difference?
@@ -1401,7 +1566,7 @@ function elements.table:shape ()
    end
 end
 
-function elements.table.output (_) end
+function elements.table:output () end
 
 local function getRadicandMode (mode)
    -- Not too sure if we should do something special/
@@ -1465,7 +1630,10 @@ function elements.sqrt:shape ()
    -- Note: In TeX, the radical sign extends a lot below the baseline,
    -- and MathML Core also has a lot of layout text about it.
    -- Not only it doesn't look good, but it's not very clear vs. OpenType.
-   local radicalGlyph = SILE.shaper:measureChar("√")
+   local radicalGlyph, found = SILE.shaper:measureChar("√")
+   if not found then
+      SU.error("Math font does not contain a square root glyph")
+   end
    local ratio = (self.radicand.height:tonumber() + self.radicand.depth:tonumber())
       / (radicalGlyph.height + radicalGlyph.depth)
    local vertAdHocOffset = (ratio > 1 and math.log(ratio) or 0) * self.radicalVerticalGap
@@ -1592,7 +1760,7 @@ function elements.padded:shape ()
    self.impadded.relY = voffset and SILE.types.length(voffset):negate() or SILE.types.length()
 end
 
-function elements.padded.output (_, _, _, _) end
+function elements.padded:output (_, _, _) end
 
 -- Bevelled fractions are not part of MathML Core, and MathML4 does not
 -- exactly specify how to compute the layout.
@@ -1652,8 +1820,6 @@ function elements.bevelledFraction:output (x, y, line)
 end
 
 elements.mathMode = mathMode
-elements.atomType = atomType
-elements.symbolDefaults = symbolDefaults
 elements.newSubscript = newSubscript
 elements.newUnderOver = newUnderOver
 
