@@ -29,7 +29,7 @@ local mirrorMaster = function (_, existing, new)
    end
 end
 
-function package.oddPage (_)
+function package:oddPage ()
    return _odd
 end
 
@@ -40,11 +40,11 @@ local function switchPage (class)
 end
 
 local _deprecate = [[
-  Directly calling master switch handling functions is no longer necessary. All
-  the SILE core classes and anything inheriting from them will take care of this
-  automatically using hooks. Custom classes that override the class:newPage()
-  function may need to handle this in other ways. By calling this hook directly
-  you are likely causing it to run twice and duplicate entries.
+   Directly calling master switch handling functions is no longer necessary. All
+   the SILE core classes and anything inheriting from them will take care of this
+   automatically using hooks. Custom classes that override the class:newPage()
+   function may need to handle this in other ways. By calling this hook directly
+   you are likely causing it to run twice and duplicate entries.
 ]]
 
 local spread_counter = 0
@@ -72,6 +72,9 @@ function package:_init (options)
    end)
    self.class:registerHook("newpage", spreadHook)
    self.class:registerHook("newpage", switchPage)
+   if not SILE.scratch.headers then
+      SILE.scratch.headers = {}
+   end
 end
 
 function package:registerCommands ()
