@@ -40,13 +40,15 @@ function shaper:shapeToken (text, options)
       end
    end
    local face = SILE.font.cache(options, self.getFace)
-   if self:checkHBProblems(text, face) then
-      return {}
-   end
    if not face then
       SU.error("Could not find requested font " .. options .. " or any suitable substitutes")
    end
-   if not options.filename and face.family ~= options.family and not substwarnings[options.family] then
+   if
+      not options.filename
+      and options.family
+      and face.family ~= options.family
+      and not substwarnings[options.family]
+   then
       substwarnings[options.family] = true
       SU.warn("Font family '" .. options.family .. "' not available, falling back to '" .. face.family .. "'")
    end
@@ -188,20 +190,4 @@ function shaper:debugVersions ()
    end
 end
 
-function shaper:checkHBProblems (text, face)
-   if hb.version_lessthan(1, 0, 4) and #text < 1 then
-      return true
-   end
-   if hb.version_lessthan(2, 3, 0) and hb.get_table(face, "CFF "):len() > 0 and not substwarnings["CFF "] then
-      SILE.status.unsupported = true
-      SU.warn([[
-         Vertical spacing of CFF fonts may be subtly inconsistent between systems
-
-         Upgrade to Harfbuzz 2.3.0 if you need absolute consistency.
-      ]])
-      substwarnings["CFF "] = true
-   end
-   return false
-end
-
 return shaper
