@@ -32,12 +32,12 @@ local warned = false
 
 function typesetter:init (frame)
    SU.deprecated("std.object", "pl.class", "0.13.0", "0.14.0", warned and "" or [[
-  The typesetter instance inheritance system for instances has been
-  refactored using a different object model. Your instance was created
-  and initialized using the object copy syntax from the stdlib model.
-  It has been shimmed for you using the new Penlight model, but this may
-  lead to unexpected behaviour. Please update your code to use the new
-  Penlight based inheritance model.]])
+      The typesetter instance inheritance system for instances has been refactored
+      using a different object model. Your instance was created and initialized
+      using the object copy syntax from the stdlib model. It has been shimmed for
+      you using the new Penlight model, but this may lead to unexpected behavior.
+      Please update your code to use the new Penlight based inheritance model.
+   ]])
    warned = true
    self:_init(frame)
 end
@@ -45,28 +45,27 @@ end
 --- Constructor
 -- @param frame A initial frame to attach the typesetter to.
 function typesetter:_init (frame)
+   SU._avoid_base_class_use(self)
    self:declareSettings()
    self.hooks = {}
    self.breadcrumbs = SU.breadcrumbs()
-
-   self.frame = nil
+   self.frame = frame
    self.stateQueue = {}
-   self:initFrame(frame)
+end
+
+function typesetter:_post_init ()
+   self:initFrame(self.frame)
    self:initState()
-   -- In case people use stdlib prototype syntax off of the instantiated typesetter...
-   getmetatable(self).__call = self.init
-   return self
 end
 
 --- Declare new setting types
-function typesetter.declareSettings (_)
+function typesetter:declareSettings ()
    -- Settings common to any typesetter instance.
    -- These shouldn't be re-declared and overwritten/reset in the typesetter
    -- constructor (see issue https://github.com/sile-typesetter/sile/issues/1708).
    -- On the other hand, it's fairly acceptable to have them made global:
    -- Any derived typesetter, whatever its implementation, should likely provide
    -- some logic for them (= widows, orphans, spacing, etc.)
-
    SILE.settings:declare({
       parameter = "typesetter.widowpenalty",
       type = "integer",
@@ -88,6 +87,13 @@ function typesetter.declareSettings (_)
       help = "Whether to ignore paragraph initial spaces",
    })
 
+   SILE.settings:declare({
+      parameter = "typesetter.brokenpenalty",
+      type = "integer",
+      default = 100,
+      help = "Penalty to be applied to broken (hyphenated) lines",
+   })
+
    SILE.settings:declare({
       parameter = "typesetter.orphanpenalty",
       type = "integer",
@@ -137,6 +143,13 @@ function typesetter.declareSettings (_)
       help = "Whether italic correction is activated or not",
    })
 
+   SILE.settings:declare({
+      parameter = "typesetter.italicCorrection.punctuation",
+      type = "boolean",
+      default = true,
+      help = "Whether italic correction is compensated on special punctuation spaces (e.g. in French)",
+   })
+
    SILE.settings:declare({
       parameter = "typesetter.softHyphen",
       type = "boolean",
@@ -179,7 +192,7 @@ function typesetter.getMargins ()
    return _margins(SILE.settings:get("document.lskip"), SILE.settings:get("document.rskip"))
 end
 
-function typesetter.setMargins (_, margins)
+function typesetter:setMargins (margins)
    SILE.settings:set("document.lskip", margins.lskip)
    SILE.settings:set("document.rskip", margins.rskip)
 end
@@ -234,26 +247,22 @@ function typesetter:pushVertical (vbox)
 end
 
 function typesetter:pushHbox (spec)
-   -- if SU.type(spec) ~= "table" then SU.warn("Please use pushHorizontal() to pass a premade node instead of a spec") end
    local ntype = SU.type(spec)
    local node = (ntype == "hbox" or ntype == "zerohbox") and spec or SILE.types.node.hbox(spec)
    return self:pushHorizontal(node)
 end
 
 function typesetter:pushUnshaped (spec)
-   -- if SU.type(spec) ~= "table" then SU.warn("Please use pushHorizontal() to pass a premade node instead of a spec") end
    local node = SU.type(spec) == "unshaped" and spec or SILE.types.node.unshaped(spec)
    return self:pushHorizontal(node)
 end
 
 function typesetter:pushGlue (spec)
-   -- if SU.type(spec) ~= "table" then SU.warn("Please use pushHorizontal() to pass a premade node instead of a spec") end
    local node = SU.type(spec) == "glue" and spec or SILE.types.node.glue(spec)
    return self:pushHorizontal(node)
 end
 
 function typesetter:pushExplicitGlue (spec)
-   -- if SU.type(spec) ~= "table" then SU.warn("Please use pushHorizontal() to pass a premade node instead of a spec") end
    local node = SU.type(spec) == "glue" and spec or SILE.types.node.glue(spec)
    node.explicit = true
    node.discardable = false
@@ -261,7 +270,6 @@ function typesetter:pushExplicitGlue (spec)
 end
 
 function typesetter:pushPenalty (spec)
-   -- if SU.type(spec) ~= "table" then SU.warn("Please use pushHorizontal() to pass a premade node instead of a spec") end
    local node = SU.type(spec) == "penalty" and spec or SILE.types.node.penalty(spec)
    return self:pushHorizontal(node)
 end
@@ -272,19 +280,16 @@ function typesetter:pushMigratingMaterial (material)
 end
 
 function typesetter:pushVbox (spec)
-   -- if SU.type(spec) ~= "table" then SU.warn("Please use pushVertical() to pass a premade node instead of a spec") end
    local node = SU.type(spec) == "vbox" and spec or SILE.types.node.vbox(spec)
    return self:pushVertical(node)
 end
 
 function typesetter:pushVglue (spec)
-   -- if SU.type(spec) ~= "table" then SU.warn("Please use pushVertical() to pass a premade node instead of a spec") end
    local node = SU.type(spec) == "vglue" and spec or SILE.types.node.vglue(spec)
    return self:pushVertical(node)
 end
 
 function typesetter:pushExplicitVglue (spec)
-   -- if SU.type(spec) ~= "table" then SU.warn("Please use pushVertical() to pass a premade node instead of a spec") end
    local node = SU.type(spec) == "vglue" and spec or SILE.types.node.vglue(spec)
    node.explicit = true
    node.discardable = false
@@ -292,7 +297,6 @@ function typesetter:pushExplicitVglue (spec)
 end
 
 function typesetter:pushVpenalty (spec)
-   -- if SU.type(spec) ~= "table" then SU.warn("Please use pushVertical() to pass a premade node instead of a spec") end
    local node = SU.type(spec) == "penalty" and spec or SILE.types.node.penalty(spec)
    return self:pushVertical(node)
 end
@@ -304,10 +308,25 @@ function typesetter:typeset (text)
       return
    end
    local pId = SILE.traceStack:pushText(text)
-   for token in SU.gtoke(text, SILE.settings:get("typesetter.parseppattern")) do
+   local parsepattern = SILE.settings:get("typesetter.parseppattern")
+   -- NOTE: Big assumption on how to guess were are in "obeylines" mode.
+   -- See https://github.com/sile-typesetter/sile/issues/2128
+   local obeylines = parsepattern == "\n"
+
+   local seenParaContent = true
+   for token in SU.gtoke(text, parsepattern) do
       if token.separator then
+         if obeylines and not seenParaContent then
+            -- In obeylines mode, each standalone line must be kept.
+            -- The zerohbox is not discardable, so it will be kept in the output,
+            -- and the baseline skip will do the rest.
+            self:pushHorizontal(SILE.types.node.zerohbox())
+         else
+            seenParaContent = false
+         end
          self:endline()
       else
+         seenParaContent = true
          if SILE.settings:get("typesetter.softHyphen") then
             local warnedshy = false
             for token2 in SU.gtoke(token.string, luautf8.char(0x00AD)) do
@@ -383,7 +402,7 @@ function speakerChangeNode:shape ()
    else
       -- Should not occur:
       -- How could it possibly be shaped differently?
-      SU.warn("Speaker change logic met an unexpected case, this might be a bug.")
+      SU.warn("Speaker change logic met an unexpected case, this might be a bug")
    end
    return node
 end
@@ -424,9 +443,15 @@ function typesetter:breakIntoLines (nodelist, breakWidth)
    return self:breakpointsToLines(breakpoints)
 end
 
+--- Extract the last shaped item from a node list.
+-- @tparam table nodelist A list of nodes.
+-- @treturn table The last shaped item.
+-- @treturn boolean Whether the list contains a glue after the last shaped item.
+-- @treturn number|nil The width of a punctuation kern after the last shaped item, if any.
 local function getLastShape (nodelist)
+   local lastShape
    local hasGlue
-   local last
+   local punctSpaceWidth
    if nodelist then
       -- The node list may contain nnodes, penalties, kern and glue
       -- We skip the latter, and retrieve the last shaped item.
@@ -434,24 +459,33 @@ local function getLastShape (nodelist)
          local n = nodelist[i]
          if n.is_nnode then
             local items = n.nodes[#n.nodes].value.items
-            last = items[#items]
+            lastShape = items[#items]
             break
          end
          if n.is_kern and n.subtype == "punctspace" then
             -- Some languages such as French insert a special space around
-            -- punctuations. In those case, we should not need italic correction.
-            break
+            -- punctuations.
+            -- In those case, we have different strategies for handling
+            -- italic correction.
+            punctSpaceWidth = n.width:tonumber()
          end
          if n.is_glue then
             hasGlue = true
          end
       end
    end
-   return last, hasGlue
+   return lastShape, hasGlue, punctSpaceWidth
 end
+
+--- Extract the first shaped item from a node list.
+-- @tparam table nodelist A list of nodes.
+-- @treturn table The first shaped item.
+-- @treturn boolean Whether the list contains a glue before the first shaped item.
+-- @treturn number|nil The width of a punctuation kern before the first shaped item, if any.
 local function getFirstShape (nodelist)
-   local first
+   local firstShape
    local hasGlue
+   local punctSpaceWidth
    if nodelist then
       -- The node list may contain nnodes, penalties, kern and glue
       -- We skip the latter, and retrieve the first shaped item.
@@ -459,58 +493,73 @@ local function getFirstShape (nodelist)
          local n = nodelist[i]
          if n.is_nnode then
             local items = n.nodes[1].value.items
-            first = items[1]
+            firstShape = items[1]
             break
          end
          if n.is_kern and n.subtype == "punctspace" then
             -- Some languages such as French insert a special space around
-            -- punctuations. In those case, we should not need italic correction.
-            break
+            -- punctuations.
+            -- In those case, we have different strategies for handling
+            -- italic correction.
+            punctSpaceWidth = n.width:tonumber()
          end
          if n.is_glue then
             hasGlue = true
          end
       end
    end
-   return first, hasGlue
+   return firstShape, hasGlue, punctSpaceWidth
 end
 
-local function fromItalicCorrection (precShape, curShape)
+--- Compute the italic correction when switching from italic to non-italic.
+-- Computing italic correction is at best heuristics.
+-- The strong assumption is that italic is slanted to the right.
+-- Thus, the part of the character that goes beyond its width is usually maximal at the top of the glyph.
+-- E.g. consider a "f", that would be the top hook extent.
+-- Pathological cases exist, such as fonts with a Q with a long tail, but these will rarely occur in usual languages.
+-- For instance, Klingon's "QaQ" might be an issue, but there's not much we can do...
+-- Another assumption is that we can distribute that extent in proportion with the next character's height.
+-- This might not work that well with non-Latin scripts.
+--
+-- @tparam table precShape The last shaped item (italic).
+-- @tparam table curShape The first shaped item (non-italic).
+-- @tparam number|nil punctSpaceWidth The width of a punctuation kern between the two items, if any.
+local function fromItalicCorrection (precShape, curShape, punctSpaceWidth)
    local xOffset
    if not curShape or not precShape then
       xOffset = 0
+   elseif precShape.height <= 0 then
+      xOffset = 0
    else
-      -- Computing italic correction is at best heuristics.
-      -- The strong assumption is that italic is slanted to the right.
-      -- Thus, the part of the character that goes beyond its width is usually
-      -- maximal at the top of the glyph.
-      -- E.g. consider a "f", that would be the top hook extent.
-      -- Pathological cases exist, such as fonts with a Q with a long tail,
-      -- but these will rarely occur in usual languages. For instance, Klingon's
-      -- "QaQ" might be an issue, but there's not much we can do...
-      -- Another assumption is that we can distribute that extent in proportion
-      -- with the next character's height.
-      -- This might not work that well with non-Latin scripts.
       local d = precShape.glyphWidth + precShape.x_bearing
       local delta = d > precShape.width and d - precShape.width or 0
       xOffset = precShape.height <= curShape.height and delta or delta * curShape.height / precShape.height
+      if punctSpaceWidth and SILE.settings:get("typesetter.italicCorrection.punctuation") then
+         xOffset = xOffset - punctSpaceWidth > 0 and (xOffset - punctSpaceWidth) or 0
+      end
    end
    return xOffset
 end
 
-local function toItalicCorrection (precShape, curShape)
-   if not SILE.settings:get("typesetter.italicCorrection") then
-      return
-   end
+--- Compute the italic correction when switching from non-italic to italic.
+-- Same assumptions as fromItalicCorrection(), but on the starting side of the glyph.
+--
+-- @tparam table precShape The last shaped item (non-italic).
+-- @tparam table curShape The first shaped item (italic).
+-- @tparam number|nil punctSpaceWidth The width of a punctuation kern between the two items, if any.
+local function toItalicCorrection (precShape, curShape, punctSpaceWidth)
    local xOffset
    if not curShape or not precShape then
       xOffset = 0
+   elseif precShape.depth <= 0 then
+      xOffset = 0
    else
-      -- Same assumptions as fromItalicCorrection(), but on the starting side of
-      -- the glyph.
       local d = curShape.x_bearing
       local delta = d < 0 and -d or 0
       xOffset = precShape.depth >= curShape.depth and delta or delta * precShape.depth / curShape.depth
+      if punctSpaceWidth and SILE.settings:get("typesetter.italicCorrection.punctuation") then
+         xOffset = punctSpaceWidth - xOffset > 0 and xOffset or 0
+      end
    end
    return xOffset
 end
@@ -526,28 +575,29 @@ local function isItalicLike (nnode)
    return font.post.italicAngle ~= 0
 end
 
-function typesetter.shapeAllNodes (_, nodelist, inplace)
+function typesetter:shapeAllNodes (nodelist, inplace)
    inplace = SU.boolean(inplace, true) -- Compatibility with earlier versions
    local newNodelist = {}
    local prec
    local precShapedNodes
+   local isItalicCorrectionEnabled = SILE.settings:get("typesetter.italicCorrection")
    for _, current in ipairs(nodelist) do
       if current.is_unshaped then
          local shapedNodes = current:shape()
 
-         if SILE.settings:get("typesetter.italicCorrection") and prec then
+         if isItalicCorrectionEnabled and prec then
             local itCorrOffset
             local isGlue
             if isItalicLike(prec) and not isItalicLike(current) then
                local precShape, precHasGlue = getLastShape(precShapedNodes)
-               local curShape, curHasGlue = getFirstShape(shapedNodes)
+               local curShape, curHasGlue, curPunctSpaceWidth = getFirstShape(shapedNodes)
                isGlue = precHasGlue or curHasGlue
-               itCorrOffset = fromItalicCorrection(precShape, curShape)
+               itCorrOffset = fromItalicCorrection(precShape, curShape, curPunctSpaceWidth)
             elseif not isItalicLike(prec) and isItalicLike(current) then
-               local precShape, precHasGlue = getLastShape(precShapedNodes)
+               local precShape, precHasGlue, precPunctSpaceWidth = getLastShape(precShapedNodes)
                local curShape, curHasGlue = getFirstShape(shapedNodes)
                isGlue = precHasGlue or curHasGlue
-               itCorrOffset = toItalicCorrection(precShape, curShape)
+               itCorrOffset = toItalicCorrection(precShape, curShape, precPunctSpaceWidth)
             end
             if itCorrOffset and itCorrOffset ~= 0 then
                -- If one of the node contains a glue (e.g. "a \em{proof} is..."),
@@ -641,6 +691,8 @@ function typesetter:boxUpNodes ()
          pageBreakPenalty = SILE.settings:get("typesetter.widowpenalty")
       elseif #lines > 1 and index == (#lines - 1) then
          pageBreakPenalty = SILE.settings:get("typesetter.orphanpenalty")
+      elseif line.is_broken then
+         pageBreakPenalty = SILE.settings:get("typesetter.brokenpenalty")
       end
       vboxes[#vboxes + 1] = self:leadingFor(vbox, self.state.previousVbox)
       vboxes[#vboxes + 1] = vbox
@@ -656,7 +708,7 @@ function typesetter:boxUpNodes ()
    return vboxes
 end
 
-function typesetter.pageTarget (_)
+function typesetter:pageTarget ()
    SU.deprecated("SILE.typesetter:pageTarget", "SILE.typesetter:getTargetLength", "0.13.0", "0.14.0")
 end
 
@@ -940,7 +992,7 @@ end
 
 function typesetter:leaveHmode (independent)
    if self.state.hmodeOnly then
-      SU.error([[Paragraphs are forbidden in restricted horizontal mode.]])
+      SU.error("Paragraphs are forbidden in restricted horizontal mode")
    end
    SU.debug("typesetter", "Leaving hmode")
    local margins = self:getMargins()
@@ -963,7 +1015,7 @@ function typesetter:inhibitLeading ()
    self.state.previousVbox = nil
 end
 
-function typesetter.leadingFor (_, vbox, previous)
+function typesetter:leadingFor (vbox, previous)
    -- Insert leading
    SU.debug("typesetter", "   Considering leading between two lines:")
    SU.debug("typesetter", "   1)", previous)
@@ -986,7 +1038,7 @@ function typesetter.leadingFor (_, vbox, previous)
    end
 end
 
--- Beggining of liner logic (contructs spanning over several lines)
+-- Beginning of liner logic (constructs spanning over several lines)
 
 -- These two special nodes are used to track the current liner entry and exit.
 -- As Sith Lords, they are always two: they are local here, so no one can
@@ -1075,7 +1127,7 @@ end
 --- Any unclosed liner is reopened on the current line, so we clone and repeat it.
 -- An assumption is that the inserts are done after the current slice content,
 -- supposed to be just before meaningful (visible) content.
--- @tparam slice slice
+-- @tparam table slice Flat nodes from current line
 -- @treturn boolean Whether a liner was reopened
 function typesetter:_repeatEnterLiners (slice)
    local m = self.state.liners
@@ -1094,7 +1146,7 @@ end
 -- Migrating content, however, must be kept outside the hboxes at top slice level.
 -- @tparam table slice Flat nodes from current line
 -- @treturn table New reboxed slice
-function typesetter._reboxLiners (_, slice)
+function typesetter:_reboxLiners (slice)
    local outSlice = {}
    local migratingList = {}
    local lboxStack = {}
@@ -1249,12 +1301,16 @@ function typesetter:breakpointsToLines (breakpoints)
             SU.debug("typesetter", "Skipping a line containing only discardable nodes")
             linestart = point.position + 1
          else
+            local is_broken = false
             if slice[#slice].is_discretionary then
                -- The line ends, with a discretionary:
                -- repeat it on the next line, so as to account for a potential postbreak.
                linestart = point.position
                -- And mark it as used as prebreak for now.
                slice[#slice]:markAsPrebreak()
+               -- We'll want a "brokenpenalty" eventually (if not an orphan or widow)
+               -- to discourage page breaking after this line.
+               is_broken = true
             else
                linestart = point.position + 1
             end
@@ -1276,7 +1332,7 @@ function typesetter:breakpointsToLines (breakpoints)
                slice = self:_reboxLiners(slice)
             end
 
-            local thisLine = { ratio = ratio, nodes = slice }
+            local thisLine = { ratio = ratio, nodes = slice, is_broken = is_broken }
             lines[#lines + 1] = thisLine
          end
       end
@@ -1289,7 +1345,7 @@ function typesetter:breakpointsToLines (breakpoints)
    return lines
 end
 
-function typesetter.computeLineRatio (_, breakwidth, slice)
+function typesetter:computeLineRatio (breakwidth, slice)
    local naturalTotals = SILE.types.length()
 
    -- From the line end, account for the margin but skip any trailing
@@ -1471,7 +1527,7 @@ end
 -- effects.
 -- If we are already in horizontal-restricted mode, the liner is processed
 -- immediately, since line breaking won't occur then.
--- @tparam string name Name of the liner (usefull for debugging)
+-- @tparam string name Name of the liner (useful for debugging)
 -- @tparam table content SILE AST to process
 -- @tparam function outputYourself Output method for wrapped boxes
 function typesetter:liner (name, content, outputYourself)
@@ -1494,4 +1550,58 @@ function typesetter:liner (name, content, outputYourself)
    end
 end
 
+--- Flatten a node list into just its string representation.
+-- @tparam table nodes Typeset nodes
+-- @treturn string Text reconstruction of the nodes
+local function _nodesToText (nodes)
+   -- A real interword space width depends on several settings (depending on variable
+   -- spaces being enabled or not, etc.), and the computation below takes that into
+   -- account.
+   local iwspc = SILE.shaper:measureSpace(SILE.font.loadDefaults({}))
+   local iwspcmin = (iwspc.length - iwspc.shrink):tonumber()
+
+   local string = ""
+   for i = 1, #nodes do
+      local node = nodes[i]
+      if node.is_nnode or node.is_unshaped then
+         string = string .. node:toText()
+      elseif node.is_glue or node.is_kern then
+         -- What we want to avoid is "small" glues or kerns to be expanded as full
+         -- spaces.
+         -- Comparing them to half of the smallest width of a possibly shrinkable
+         -- interword space is fairly fragile and empirical: the content could contain
+         -- font changes, so the comparison is wrong in the general case.
+         -- It's a simplistic approach. We cannot really be sure what a "space" meant
+         -- at the point where the kern or glue got absolutized.
+         if node.width:tonumber() > iwspcmin * 0.5 then
+            string = string .. " "
+         end
+      elseif not (node.is_zerohbox or node.is_migrating) then
+         -- Here, typically, the main case is an hbox.
+         -- Even if extracting its content could be possible in some regular cases
+         -- we cannot take a general decision, as it is a versatile object  and its
+         -- outputYourself() method could moreover have been redefined to do fancy
+         -- things. Better warn and skip.
+         SU.warn("Some content could not be converted to text: " .. node)
+      end
+   end
+   -- Trim leading and trailing spaces, and simplify internal spaces.
+   return pl.stringx.strip(string):gsub("%s%s+", " ")
+end
+
+--- Convert a SILE AST to a textual representation.
+-- This is similar to SU.ast.contentToString(), but it performs a full
+-- typesetting of the content, and then reconstructs the text from the
+-- typeset nodes.
+-- @tparam table content SILE AST to process
+-- @treturn string Textual representation of the content
+function typesetter:contentToText (content)
+   self:pushState()
+   self.state.hmodeOnly = true
+   SILE.process(content)
+   local text = _nodesToText(self.state.nodes)
+   self:popState()
+   return text
+end
+
 return typesetter
